let img
let table
let manager
let runBool = false

function preload() {
  table = loadTable('data.csv', 'header')
  img = loadImage('library_bw.jpg')
}

function setup() {
  createCanvas(950, 650)
  noStroke()
  textSize(10)
  frameRate(25)

  manager = new Manager()
  background(255)
  image(img, -1, 0)
  manager.run()
}

function draw() {
  if (runBool) {
    background(255)
    image(img, -1, 0)
    manager.run()
  }
}

function mousePressed() {
  if (mouseX > 80 && mouseX < 255 && mouseY > 60 && mouseY < 110) {
    runBool = !runBool
    manager.button()
  }
}

function mouseMoved() {
  if (mouseX > 80 && mouseX < 255 && mouseY > 60 && mouseY < 110) {
    cursor(HAND);
  } else {
    cursor(ARROW);
  }
}
 
// ------------------------------------------------------- \\



let pings = []
let beacons = []
let i = 0

class Manager {
  constructor() {
    this.addBeacons()
  }

  addBeacons() {
    beacons.push(new Beacon(87, 170, "b3001")); // beacon 1
    beacons.push(new Beacon(181, 70, "b3002")); // beacon 2
    beacons.push(new Beacon(264, 70, "b3003")); // beacon 3
    beacons.push(new Beacon(356, 70, "b3004")); // beacon 4
    beacons.push(new Beacon(180, 135, "b3005")); // beacon 5
    beacons.push(new Beacon(265, 135, "b3006")); // beacon 6
    beacons.push(new Beacon(355, 135, "b3007")); // beacon 7
    beacons.push(new Beacon(180, 197, "b3008")); // beacon 8
    beacons.push(new Beacon(60, 304, "b3009")); // beacon 9
    beacons.push(new Beacon(178, 308, "b3010")); // beacon 10
    beacons.push(new Beacon(261, 308, "b3011")); // beacon 11
    beacons.push(new Beacon(348, 308, "b3012")); // beacon 12
    beacons.push(new Beacon(450, 308, "b3013")); // beacon 13
  }

  addPing(x, y) {
    pings.push(new Ping(x, y))
  }

  legenda() {
    textSize(12)
    push()
    translate(300, 50)
    fill(0);
    text(table.getRow(i).getString("date"), 7, 15); // draw timestamp
    strokeWeight(1);
    stroke(0);
    fill(250, 140, 0);
    circle(15, 30, 10); // draw beacon
    noStroke()
    fill(0);
    text("BLE beacon", 25, 34);
    fill(0, 150, 200);
    circle(15, 45, 12); // draw ping
    fill(0, 100, 255);
    circle(15, 45, 5); // draw front ping red
    fill(0);
    text("current location", 25, 49);
    fill(250, 140, 0, 75);
    circle(15, 60, 12); // draw front ping red
    fill(0);
    text("signal strength", 25, 64);
    pop()
  }

  button() {
    if (runBool) {
      fill(0, 150, 210)
      rect(80, 60, 175, 50, 20)
      fill(0)
      textSize(18)
      text('Pause simulation', 100, 90)
    } else {
      fill(0, 150, 210)
      rect(80, 60, 175, 50, 20)
      fill(0)
      textSize(18)
      text('Start simulation', 100, 90)
    }
  }

  run() {
    push()
    translate(120, 185)

    if (i < table.getRowCount() - 1) {
      var x = table.getRow(i).getNum('hor') * 20
      var y = table.getRow(i).getNum('ver') * 20
      this.addPing(x, y)
      i++
    } else {
      i = 0
    }

    for (let k = 0; k < beacons.length; k++) {
      beacons[k].render(i, x, y)
    }

    for (let j = 0; j < pings.length; j++) {
      pings[j].render()
      if (pings[j].lifetime <= 0) {
        pings.slice(j, 1)
      }
    }

    pop()

    this.legenda()
    this.button()
  }
}

class Beacon {
  constructor(x, y, name) {
    this.position = createVector(x, y)
    this.beaconName = name
    this.radius = 10
    this.row = ''
  }

  render(ping, pingx, pingy) {
    let rssi = table.getRow(ping).getNum(this.beaconName)
    if (rssi > -100) {
      let radius = dist(this.position.x, this.position.y, pingx, pingy) * 2
      strokeWeight(0.2)
      stroke(0, 100, 0)
      fill(250, 140, 0, map(radius, 0, 250, 150, 25))
      circle(this.position.x, this.position.y, radius)
    }

    strokeWeight(1)
    stroke(0)
    fill(250, 140, 0)
    circle(this.position.x, this.position.y, this.radius)
  }
}

class Ping {
  constructor(x, y) {
    this.position = createVector(x, y)
    this.lifetime = 250
  }

  render() {
    noStroke()
    fill(0, 150, 200, map(this.lifetime, 250, 0, 255, -100))
    circle(this.position.x, this.position.y, map(this.lifetime, 250, 0, 15, 0))

    if (this.lifetime == 250) {
      fill(0, 100, 255)
      circle(this.position.x, this.position.y, 5)
    }

    this.lifetime -= 2
  }
}